# This file is part of SSHare.
#
# SSHare is free software: you can redistribute it and/or modify it under the terms of
# the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# SSHare is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# SSHare. If not, see <https://www.gnu.org/licenses/>.

class NoDefault: pass

class NoArgument: pass
class Flag: pass

class Argument:
    def __init__(self,
                 short=None,
                 long=None,
                 action=None,
                 nargs=None,
                 const=None,
                 default=NoArgument,
                 type=None,
                 choices=None,
                 required=None,
                 help=None):
        self.short   = short
        self.long    = long
        self.action  = action
        self.nargs   = nargs
        self.const   = const
        self.default = default
        self.type    = type
        self.choices = choices
        self.help    = help

    def is_valid(self):
        return (self.short != None and self.short != "") or (self.long != None and self.long != "")

    def pretty(self):
        if self.short and self.long:
            pretty = f"-{self.short}, --{self.long}"
        elif self.long:
            pretty = f"--{self.long}"
        else:
            pretty = f"-{self.short}"
        return pretty + f" {self.help}"

    def bind(self, plugin, argument):
        self.plugin  = plugin
        self.metavar = argument
        self.dest    = f"{plugin.name}_{argument}"

    def add(self, parser, used_arguments):
        keywords = [
            "action",
            "nargs",
            "const",
            "default",
            "type",
            "choices",
            "help",
            "metavar",
            "dest"
        ]
        kwargs = {}
        for keyword in keywords:
            value = getattr(self, keyword)
            if value != None:
                kwargs[keyword] = value
        parser.add_argument(
            f"-{self.short}",
            f"--{self.long}",
            **kwargs
        )
        if self.short:
            used_arguments["short"] = self.plugin
        if self.long:
            used_arguments["long"] = self.plugin
